import re
import json
import platform
import threading
from typing import List
from urllib.parse import urljoin
from urllib.parse import urlparse

import allure
import requests
from box import Box
import json as built_json
from jsonpath import jsonpath as jp
from requests.models import Response
from framework.extract import Extract
from framework.utils.log_util import logger
from framework.global_attribute import CONTEXT
from requests_toolbelt import MultipartEncoder
from framework.models.flows import Flows, get_mysql_conn

mysql_session = get_mysql_conn()


class ResponseUtil(object):
    def __init__(self, response: Response):
        self.response = response
        self.box = Box(response.json())

    def __getattr__(self, name):
        try:
            return getattr(self.box, name)
        except AttributeError:
            return getattr(self.response, name)

    def __str__(self):
        return self.text

    def jsonpath(self, expr, headers=False):
        """
        使用jsonpath语法从响应体中提取内容
        @param expr: jsonpath语法
        @param headers: 从响应头中提取内容
        @return:
        """
        result = jp(dict(self.response.headers), expr) if headers else jp(self.json(), expr)
        if result is False:
            return ""
        if len(result) == 1:
            return result[0]
        return result

    @property
    def jsonp(self):
        """返回json"""
        jsonp_re = re.compile(r".*?\((?P<text>.*)\)")
        return built_json.loads(jsonp_re.match(self.response.text).group("text"))

    def json(self):
        return self.response.json()

    def extract(self, app, key, expression):
        """
        进行变量提取
        :param app:
        :param key: 保存变量的名称
        :param expression: 提取变量表达式
        :return:
        """
        Extract(self.response, app).extract(key, expression)

    def extracts(self, app, expressions: List[dict]):
        """
        进行变量提取
        :param app:
        :param expressions: 提取变量表达式
        :return:
        """
        Extract(self.response, app).extracts(expressions)


class HttpClient(object):

    def __init__(self, headers=None):
        """

        @param headers: 请求头
        @param cookies: cookies
        """

        self.headers = {
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/130.0.0.0 Safari/537.36",
            "Content-Type": "application/json"
        }
        self.headers.update(headers or {})
        self.response = None

    def __send_request(self, data):
        _is_multipart = data.get("_is_multipart")
        request_obj = data.get("request")
        with allure.step("发送请求"):
            if _is_multipart:
                m = MultipartEncoder(fields=request_obj["data"])
                request_obj["data"] = m
                request_obj["headers"] = {'Content-Type': m.content_type}
            else:
                request_obj["headers"].update(self.headers)
            self.response = ResponseUtil(requests.request(**request_obj))

            with allure.step(f"请求url: {request_obj.get('url')}"):
                logger.info(f"请求url: {request_obj.get('url')}")
            with allure.step(f"请求method: {request_obj.get('method')}"):
                logger.info(f"请求method: {request_obj.get('method')}")
            with allure.step(f"请求headers: {request_obj.get('headers')}"):
                logger.info(f"请求headers: {request_obj.get('headers')}")
            if request_obj.get('request'):
                with allure.step(f"请求参数params: {request_obj.get('params')}"):
                    logger.info(f"请求参数params: {request_obj.get('params')}")
            if request_obj.get('data'):
                with allure.step(f"请求参数data: {request_obj.get('data')}"):
                    logger.info(f"请求参数data: {request_obj.get('data')}")
            if request_obj.get('json'):
                with allure.step(f"请求参数json: {request_obj.get('json')}"):
                    logger.info(f"请求参数json: {request_obj.get('json')}")

        with allure.step("响应结果"):
            with allure.step(f"响应状态码: {self.response.status_code}"):
                logger.info(f"响应状态码: {self.response.status_code}")
            with allure.step(f"响应headers: {self.response.headers}"):
                logger.info(f"响应headers: {self.response.headers}")
            with allure.step(f"响应body: {self.response.json()}"):
                logger.info(f"响应body: {self.response.json()}")

        # # 断言
        # validates = params.get("validate")
        # if validates:
        #     with allure.step("结果断言"):
        #         Validate(self.response).valid(validates)

        # 提取变量
        expressions = data.get("extract")
        belong_app = data.get("_belong_app")
        if expressions:
            with allure.step("提取变量"):
                Extract(self.response, belong_app).extracts(expressions)

        if platform.platform() == "Linux":
            try:
                # 请求信息存表
                thread = threading.Thread(target=self.save_request, args=(data,))
                thread.start()
            except Exception as e:
                logger.error(f"抓取接口异常:{e}")
        return self.response

    def request(self, *args, app, **kwargs):
        if args and isinstance(args[0], dict) and args[0].get("request") is not None:
            return self.__send_request(args[0])
        if args and isinstance(args[1], str):
            args = list(args)
            args[1] = urljoin(CONTEXT.get(app=app, key="domain"), args[1])
        else:
            kwargs["url"] = urljoin(CONTEXT.get(app=app, key="domain"), kwargs["url"])
        return ResponseUtil(requests.request(*args, headers=self.headers, **kwargs))

    def post(self, app, url, data=None, json=None, **kwargs):
        return self.request("post", urljoin(CONTEXT.get(app=app, key="domain"), url), app=app, data=data, json=json
                            , **kwargs)

    def get(self, app, url, params=None, **kwargs):
        return self.request("get", urljoin(CONTEXT.get(app=app, key="domain"), url), app=app, params=params, **kwargs)

    def put(self, app, url, data=None, **kwargs):
        return self.request("put", urljoin(CONTEXT.get(app=app, key="domain"), url), app=app, data=data, **kwargs)

    def delete(self, app, url, **kwargs):
        return self.request("delete", urljoin(CONTEXT.get(app=app, key="domain"), url), app=app, **kwargs)

    def update_headers(self, headers):
        self.headers.update(headers)

    @staticmethod
    def save_request(params):
        request = params.get("request")
        parsed_url = urlparse(request.get("url"))
        url = parsed_url.path
        method = request.get("method")
        flow_obj = mysql_session.query(Flows).filter_by(req_url=url).first()
        if not flow_obj:
            params = request.get("params", None)
            _json = request.get("json", None)
            _data = request.get("data", None)
            flow = Flows(
                req_method=method,
                req_url=url,
                req_query=json.dumps(params) if params else None,
                req_body=json.dumps(_json) if _json else None or json.dumps(_data) if _data else None
            )
            mysql_session.add(flow)
            mysql_session.commit()
